import { has } from './has.js'
import { is } from './is.js'

const _locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ language,
  /** @type {string} */ text,
  /** @type {string} */ version
) => {
  const l = locales?.[language]
  // @ts-expect-error
  const v = l?.[version]?.[text]

  if (is(String, v)) {
    return v
  }

  const t = l?.[text]

  return is(String, t) ? t : text
}

/**
 * Language translations helper.
 */
export const locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ defaultLanguage,
  languages = navigator.languages
) => {
  for (const language of languages) {
    if (has(language, locales)) {
      // @ts-expect-error
      return _locale.bind(0, locales, language)
    }
  }

  // @ts-expect-error
  return _locale.bind(0, locales, defaultLanguage)
}

export const /** @type {Record<string, () => void>} */ tests = {
  locale: () => {
    const locales = {
      pl: {
        Password: 'Hasło',
        button: { Login: 'Zaloguj' }
      }
    }
    const _ = locale(locales, 'pl', [])

    console.assert(_('Login') === 'Login')
    console.assert(_('Password') === 'Hasło')

    console.assert(_('Undefined text') === 'Undefined text')

    console.assert(_('Login', 'button') === 'Zaloguj')

    console.assert(_('Password', 'undefined_version') === 'Hasło')
    console.assert(_('Undefined text', 'undefined_version') === 'Undefined text')

    console.assert(_('toString') === 'toString')
    console.assert(_('toString', 'undefined_version') === 'toString')
  }
}
